#include "FieldModuleIndicator.h"

FieldModuleIndicator::FieldModuleIndicator(QWidget* parent) : 
	QWidget(parent) {
	setMinimumHeight(30);
	setMinimumWidth(200);
	
	tFN = 0.0f;
	cFN = 0.0f;
}

void FieldModuleIndicator::updateFieldModule(float m, Eigen::Vector3f b)
{
	if (m < 0 || m > 1.0e4f) return; 

	float d = fabs(m - b.norm());
	cFN = 0.9 * cFN + 0.1 * m; 
	
	if (cFN > 100) {
		tFN = 1.0;
	} else {
		tFN = cFN / 100.0f;
	}
	
	update();
}

#define EPSILON_INCLINATION 10.0f
#define EPSILON_MODULE 10.0f

void FieldModuleIndicator::paintEvent(QPaintEvent*)
{
	QPainter painter(this);

	QFont f;
	f.setPixelSize(18);
	painter.setFont(f);
	
	float sFN = tFN * (width()-100-7);
	float col = tFN * 255;
	
	QPen white(QColor(255, 255, 255));
	QPen black(QColor(80, 80, 80));
	QPen red(QColor(255, 150, 150));
	QBrush redB(QColor(255, 150, 150));
	QPen blue(QColor(150, 150, 255));
	QBrush blueB(QColor(150, 150, 255));
	QBrush whiteB(QColor(240, 240, 240));
	
	painter.setPen(black);
	painter.setBrush(whiteB);
	painter.drawText(0, height()/2, QString("Magnetic"));
	painter.drawText(0, height()/2+20, QString("noise:"));
	painter.drawRect(100, 0, width()-100-5, height()-5);		

	/* if (cFN > (EPSILON_INCLINATION + EPSILON_MODULE)) {
		painter.setPen(red);	
		painter.setBrush(redB);
	} else { */
		painter.setPen(blue);	
		painter.setBrush(blueB);
	// }
	painter.drawRect(101, 1, (int) sFN, height()-2-5);	

	painter.setPen(black);	
	painter.drawText(150, height()/2+5, QString("weak"));		
	painter.drawText(width()-100, height()/2+5, QString("strong"));	
}