/****************************************************************************
**
** Copyright (C) 2011 LP-Research
** All rights reserved.
** Contact: LP-Research (klaus@lp-research.com)
**
** This file is part of the Open Motion Analysis Toolkit (OpenMAT).
**
** OpenMAT is free software: you can redistribute it and/or modify it under 
** the terms of the GNU General Public License as published by the Free 
** Software Foundation, either version 3 of the License, or (at your option) 
** any later version.
** 
** OpenMAT is distributed in the hope that it will be useful, but WITHOUT 
** ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
** FITNESS FOR A PARTICULAR PURPOSE. See the GNU \ General Public License 
** for more details.
** 
** You should have received a copy of the GNU General Public License along 
** with the OpenMAT library. If not, see <http://www.gnu.org/licenses/>.
**
****************************************************************************/

#include "GraphWindow.h"

#include <iostream>
#include <string>

using namespace std;

GraphWindow::GraphWindow(QWidget* parent) : QWidget(parent)
{
    accGraph = new Plot("Accelerometer (g)", "No. samples", "", 0, 128, 3, -8.0f, 8.0f);	
    gyroGraph = new Plot("Gyroscope (deg. / s)", "No. samples", "", 0, 128, 3, -2000.0f, 2000.0f);	
    magGraph = new Plot("Magnetometer (uT)", "No. samples", "", 0, 128, 3, -200, 200);	
    quaternionGraph = new Plot("Quaternion (quaternion units)", "No. samples", "", 0, 128, 4, -1.0f, 1.0f);	
    angleGraph = new Plot("Angle (degree)", "No. samples", "", 0, 128, 3, -181.0f, 181.0f);	
	pressureGraph = new Plot("Altitude (m)", "No. samples", "", 0, 128, 1, 50.0f, 100.0f);	
	temperatureGraph = new Plot("Temperature (deg. C)", "No. samples", "", 0, 128, 1, 0.0f, 50.0f);
    linAccGraph = new Plot("Linear acceleration (g)", "No. samples", "", 0, 128, 3, -0.5f, 0.5f);
	
	QVBoxLayout *vLayout = new QVBoxLayout();
	vLayout->addWidget(quaternionGraph);		
	vLayout->addWidget(angleGraph);
	vLayout->addWidget(pressureGraph);
	vLayout->addWidget(accGraph);
	vLayout->addWidget(gyroGraph);
	vLayout->addWidget(magGraph);
	vLayout->addWidget(linAccGraph);
	this->setLayout(vLayout);
	
	for (int i=0; i<7; i++) vLayout->setStretch(i, 1);
	
    setAutoFillBackground(true);
    setPalette(QPalette(QColor(255, 255, 255)));

	setMode(GRAPH_MODE_RAW);
}

void GraphWindow::setMode(int mode)
{
	switch (mode) {
	case GRAPH_MODE_RAW:
		quaternionGraph->hide();
		angleGraph->hide();
		accGraph->show();
		gyroGraph->show();
		magGraph->show();
		pressureGraph->hide();
		linAccGraph->hide();
		break;
		
	case GRAPH_MODE_ORIENTATION:
		quaternionGraph->show();
		angleGraph->show();
		linAccGraph->show();
		accGraph->hide();
		gyroGraph->hide();
		magGraph->hide();
		pressureGraph->hide();
		break;
		
	case GRAPH_MODE_PRESSURE:		
		quaternionGraph->hide();
		angleGraph->hide();
		accGraph->hide();
		gyroGraph->hide();
		magGraph->hide();
		pressureGraph->show();
		linAccGraph->hide();
	}	
}

void GraphWindow::plotDataSet(ImuData ld)
{
	accGraph->addData(0, ld.a[0]);
	accGraph->addData(1, ld.a[1]);
	accGraph->addData(2, ld.a[2]);
	gyroGraph->addData(0, ld.g[0]);
	gyroGraph->addData(1, ld.g[1]);
	gyroGraph->addData(2, ld.g[2]);
	magGraph->addData(0, ld.b[0]);
	magGraph->addData(1, ld.b[1]);
	magGraph->addData(2, ld.b[2]);
	quaternionGraph->addData(0, ld.q[0]);
	quaternionGraph->addData(1, ld.q[1]);
	quaternionGraph->addData(2, ld.q[2]);
	quaternionGraph->addData(3, ld.q[3]);	
	angleGraph->addData(0, ld.r[0]);
	angleGraph->addData(1, ld.r[1]);
	angleGraph->addData(2, ld.r[2]);
	pressureGraph->addData(0, ld.p);
	linAccGraph->addData(0, ld.linAcc[0]);
	linAccGraph->addData(1, ld.linAcc[1]);
	linAccGraph->addData(2, ld.linAcc[2]);
}

void GraphWindow::clearGraphs(void)
{
	accGraph->clearData();
	gyroGraph->clearData();
	magGraph->clearData();
	quaternionGraph->clearData();
	angleGraph->clearData();
	pressureGraph->clearData();
	linAccGraph->clearData();
}

void GraphWindow::setActiveLpms(int openMatId)
{
	activeOpenMatId = openMatId;
}