/****************************************************************************
**
** Copyright (C) 2011 LP-Research
** All rights reserved.
** Contact: LP-Research (klaus@lp-research.com)
**
** This file is part of the Open Motion Analysis Toolkit (OpenMAT).
**
** OpenMAT is free software: you can redistribute it and/or modify it under 
** the terms of the GNU General Public License as published by the Free 
** Software Foundation, either version 3 of the License, or (at your option) 
** any later version.
** 
** OpenMAT is distributed in the hope that it will be useful, but WITHOUT 
** ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
** FITNESS FOR A PARTICULAR PURPOSE. See the GNU \ General Public License 
** for more details.
** 
** You should have received a copy of the GNU General Public License along 
** with the OpenMAT library. If not, see <http://www.gnu.org/licenses/>.
**
****************************************************************************/

#ifndef GRAPH_WINDOW
#define GRAPH_WINDOW

#include <QWidget>
#include <QVBoxLayout>
#include <QLabel>
#include <QPushButton>
#include <QHBoxLayout>
#include <QVBoxLayout>
#include <QComboBox>
#include <QGroupBox>
#include <QPalette>

#include "Plot.h"
#include "ImuData.h"

#define GRAPH_MODE_RAW 0
#define GRAPH_MODE_ORIENTATION 1
#define GRAPH_MODE_PRESSURE 2

/*!
	\brief	Contains the sensor data plots.

	The following plots are shown: 
		- accelerometer data (x, y, z axis)
		- gyroscope data (x, y, z axis)
		- magnetometer (x, y, z axis)
		- quaternion orientation data (w, x, y, z)
*/		
class GraphWindow : public QWidget
{
Q_OBJECT
	
public:
	GraphWindow(QWidget *parent = 0);
	void clearGraphs(void);
	void setActiveLpms(int openMatId);
	void setMode(int m);
	
public slots:
	void plotDataSet(ImuData ld);
	
public:
	Plot *accGraph;	
    Plot *gyroGraph;	
    Plot *magGraph;	
    Plot *angleGraph;	    
	Plot *quaternionGraph;
	Plot *pressureGraph;
	Plot *temperatureGraph;
	Plot *linAccGraph;
	
	int activeOpenMatId;
	int mode;
};

#endif