/****************************************************************************
**
** Copyright (C) 2011 LP-Research
** All rights reserved.
** Contact: LP-Research (klaus@lp-research.com)
**
** This file is part of the Open Motion Analysis Toolkit (OpenMAT).
**
** OpenMAT is free software: you can redistribute it and/or modify it under 
** the terms of the GNU General Public License as published by the Free 
** Software Foundation, either version 3 of the License, or (at your option) 
** any later version.
** 
** OpenMAT is distributed in the hope that it will be useful, but WITHOUT 
** ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
** FITNESS FOR A PARTICULAR PURPOSE. See the GNU \ General Public License 
** for more details.
** 
** You should have received a copy of the GNU General Public License along 
** with the OpenMAT library. If not, see <http://www.gnu.org/licenses/>.
**
****************************************************************************/

#include "Plot.h"

#include <iostream>

Plot::Plot(string title, string xAxis, string yAxis, int color, int maxData, int nCurves, float yMax, float yMin, QWidget *parent) 
	: QwtPlot(parent)
{
	// (void) new QwtPlotPanner(canvas());
	// (void) new QwtPlotMagnifier(canvas());
	
	// setMinimumSize(100, 150);	
	
	this->nCurves = nCurves;
	
	if (maxData < MAX_DATA)
	{
		this->maxData = maxData;
	} else {
		this->maxData = MAX_DATA;
	}

    setAutoFillBackground(true);
    setPalette(QPalette(QColor(255, 255, 255)));

    insertLegend(new QwtLegend(), QwtPlot::RightLegend);
	
	QwtText qwtTitle(title.c_str());

	QFont f;
	f.setUnderline(true);
	f.setPointSize(11);
	f.setBold(false);	

	qwtTitle.setFont(f);
	setTitle(qwtTitle);
	
	QwtText t(QString(xAxis.c_str()));
	f.setUnderline(false);	
	t.setFont(f);	
    setAxisTitle(xBottom, t);
    setAxisScale(xBottom, 0.0, (float) maxData);

    setAxisScale(yLeft, yMax, yMin);	
	setAxisMaxMajor(yLeft, 5);
	// setAxisAutoScale(yLeft);
	
    // setAxisScale(yLeft, -5000, 5000);
  	
	/* QwtPlot::setFrameStyle(QFrame::NoFrame);
  	QwtPlot::setLineWidth(0);
  	QwtPlot::setCanvasLineWidth(2); */
  	QwtPlot::setAutoReplot();

    canvas()->setLineWidth(1);
	// canvas()->setFrameStyle(QFrame::NoFrame);
    canvas()->setFrameStyle(QFrame::Box | QFrame::Plain);

    QPalette canvasPalette(Qt::white);
    canvasPalette.setColor(QPalette::Foreground, QColor(0, 0, 0));
    canvas()->setPalette(canvasPalette); 
	
	QwtScaleWidget *scaleWidget = axisWidget(QwtPlot::yLeft);
	scaleWidget->scaleDraw()->setMinimumExtent(50);	

	QwtPlotGrid *grid = new QwtPlotGrid;
    grid->enableX(true);
    grid->enableY(false);
    grid->setMajPen(QPen(Qt::gray, 0, Qt::DotLine));
    grid->setMinPen(QPen(Qt::gray, 0, Qt::DotLine));
    grid->attach(this);
	
	/* QPalette p(palette());
    canvasPalette.setColor(backgroundRole(), Qt::white);
    canvas()->setPalette(p); */
	
	std::string curveName[4];
	curveName[0] = std::string("X-Axis");
	curveName[1] = std::string("Y-Axis");
	curveName[2] = std::string("Z-Axis");
	curveName[3] = std::string("W-Axis");	

	for (int i=0; i<nCurves; i++)
	{
		Curve c;
		
		c.qwtCurve = new QwtPlotCurve(curveName[i].c_str());
		c.nData = 0;
		c.xPos = 0;
		c.first = true;

		switch (i)
		{
		case 0:
			c.qwtCurve->setPen(QPen(Qt::red));
			break;

		case 1:
			c.qwtCurve->setPen(QPen(Qt::green));
			break;
			
		case 2:
			c.qwtCurve->setPen(QPen(Qt::blue));
			break;

		default:
			c.qwtCurve->setPen(QPen(Qt::magenta));
			break;
		}
		
		c.qwtCurve->attach(this);
		c.qwtCurve->setRawSamples(c.xData, c.yData, c.nData); 
		
		curves.push_back(c);
	}

	setXMarker("");
	
	clearData();
}

void Plot::addData(int i, float y)
{	
	if (i < nCurves) {
		if (curves[i].xPos /*curves[i].nData*/ > maxData-1) {
			// curves[i].nData = 0;
			curves[i].xPos = 0;
			curves[i].nData = maxData;
			curves[i].first = false;
			// clearXMarker();
		}

		if (curves[i].first == true) curves[i].nData = curves[i].xPos+1;
	
		curves[i].xData[curves[i].xPos /*curves[i].nData*/] = (float) curves[i].xPos; // curves[i].nData;
		curves[i].yData[curves[i].xPos /*curves[i].nData*/] = (float) y;
				
		setData(i);

		if (markerX.size() > 0) markerX[0]->setXValue((float) curves[i].xPos);

		++curves[i].xPos;
		// ++curves[i].nData;
	}
}

void Plot::setXMarker(string text)
{	
	QwtPlotMarker* m;
	
    m = new QwtPlotMarker();
    //m->setLabel(QString::fromStdString(text));
    m->setLabelAlignment(Qt::AlignRight | Qt::AlignTop);
	//m->setLabelOrientation(Qt::Horizontal);
    m->setLineStyle(QwtPlotMarker::VLine);
    m->setLinePen(QPen(Qt::gray, 0, Qt::SolidLine));
    m->setXValue(0); //curves[0].nData);

    m->attach(this);

	markerX.push_back(m);
}

void Plot::clearXMarker(void)
{
	for (unsigned int i=0; (int) i<markerX.size(); i++)
	{
		markerX[i]->detach();
		delete markerX[i];
	}
	
	markerX.clear();
}

void Plot::setData(int i)
{
	if (i<nCurves)
	{	
		curves[i].qwtCurve->setRawSamples(curves[i].xData, curves[i].yData, curves[i].nData); 
	}
}

void Plot::clearData(void)
{
	for (int i=0; i<nCurves; i++)
	{
		curves[i].nData = 0; // maxData;
		curves[i].first = true;

		for (int j=0; j<maxData; j++) {
			curves[i].xData[j] = 0.0f;
			curves[i].yData[j] = 0.0f;
		}

		curves[i].nData = 0;
		curves[i].xPos = 0;
	
		curves[i].qwtCurve->setRawSamples(curves[i].xData, curves[i].yData, curves[i].nData); 	
	}
	
	// clearXMarker();
}