/****************************************************************************
**
** Copyright (C) 2011 LP-Research
** All rights reserved.
** Contact: LP-Research (klaus@lp-research.com)
**
** This file is part of the Open Motion Analysis Toolkit (OpenMAT).
**
** OpenMAT is free software: you can redistribute it and/or modify it under 
** the terms of the GNU General Public License as published by the Free 
** Software Foundation, either version 3 of the License, or (at your option) 
** any later version.
** 
** OpenMAT is distributed in the hope that it will be useful, but WITHOUT 
** ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
** FITNESS FOR A PARTICULAR PURPOSE. See the GNU \ General Public License 
** for more details.
** 
** You should have received a copy of the GNU General Public License along 
** with the OpenMAT library. If not, see <http://www.gnu.org/licenses/>.
**
****************************************************************************/

#ifndef SINUS_PLOT
#define SINUS_PLOT

#include <QWidget>
#include <QFont>

#include <qwt_plot.h>
#include <qwt_plot_marker.h>
#include <qwt_plot_curve.h>
#include <qwt_legend.h>
#include <qwt_series_data.h>
#include <qwt_plot_canvas.h>
#include <qwt_plot_grid.h>
#include <qwt_plot_panner.h>
#include <qwt_plot_magnifier.h>
#include <qwt_text.h>
#include <qwt_text_label.h>
#include <qwt_math.h>
#include <qwt_scale_widget.h>
//#include <qwt_scale_draw.h>

#include <string>

using namespace std;

#define MAX_DATA 512

#define STD_SPACING 5

/*!
	\brief	Contains all information for one data graph curve.
*/
class Curve 
{
public:
	QwtPlotCurve *qwtCurve;
	int nData;
	double xData[MAX_DATA];
	double yData[MAX_DATA];
	int xPos;
	bool first;
};

/*!
	\brief	QwtPlot window that contains a data graph.
	
	This class uses the QWT plotting library. For further documentation,
	please refer to http://qwt.sourceforge.net/.
*/
class Plot : public QwtPlot
{
Q_OBJECT	
	
public:
    Plot(string title, string xAxis, string yAxis, int color, int maxData, int nCurves,  float yMax, float yMin, QWidget *parent = NULL);
	void addData(int i, float y);
	void clearData(void);
	void setData(int i);
	void setXMarker(string text);
	void clearXMarker(void);
	
public:
	vector<Curve> curves;
	int maxData;
	int nCurves;
	vector<QwtPlotMarker *> markerX;
};

#endif