/***********************************************************************
** Copyright (C) 2011 LP-Research
** All rights reserved.
** Contact: LP-Research (klaus@lp-research.com)
**
** This file is part of the Open Motion Analysis Toolkit (OpenMAT).
**
** Redistribution and use in source and binary forms, with 
** or without modification, are permitted provided that the 
** following conditions are met:
**
** Redistributions of source code must retain the above copyright 
** notice, this list of conditions and the following disclaimer.
** Redistributions in binary form must reproduce the above copyright 
** notice, this list of conditions and the following disclaimer in 
** the documentation and/or other materials provided with the 
** distribution.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
** FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
** LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
** DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
** THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
** OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
***********************************************************************/

#include "SensorGuiContainer.h"

SensorGuiContainer::SensorGuiContainer(LpmsSensorI* sensor, QTreeWidget* tree) :
	sensor(sensor) {
	int l;
	
	deviceType = 0;
	sensor->getConfigurationPrm(PRM_DEVICE_TYPE, &deviceType);	
	
	treeItem = new QTreeWidgetItem(tree, QStringList(QString("Sensor")));	
	QTreeWidgetItem* subTreeItem = new QTreeWidgetItem(treeItem);	
	subTreeItem->setFlags(Qt::ItemIsEnabled);
	
	QGridLayout* gl = new QGridLayout();

	l = 0;
	gl->addWidget(new QLabel("Connection:"), l, 0); ++l;
	gl->addWidget(new QLabel("Sensor status:"), l, 0); ++l;
	gl->addWidget(new QLabel("Device ID:"), l, 0); ++l;
	gl->addWidget(new QLabel("Firmware Version:"), l, 0); ++l;
	gl->addWidget(new QLabel("IMU ID:"), l, 0); ++l;
	gl->addWidget(new QLabel("Selected TX data:"), l, 0); ++l; ++l; ++l; ++l; ++l; ++l;	
	gl->addWidget(new QLabel("Sampling rate:"), l, 0); ++l;
	gl->addWidget(new QLabel("GYR range:"), l, 0); ++l;
	gl->addWidget(new QLabel("ACC range:"), l, 0); ++l;
	gl->addWidget(new QLabel("MAG range:"), l, 0); ++l;
	gl->addWidget(new QLabel("Mag. correction:"), l, 0); ++l;
	gl->addWidget(new QLabel("Filter mode:"), l, 0); ++l;
	gl->addWidget(new QLabel("Gyr. thresh.:"), l, 0); ++l;
	gl->addWidget(new QLabel("Gyr. autocalibration:"), l, 0); ++l;	
		
	l = 0;
	gl->addWidget(statusItem = new QLabel(""), l, 1); ++l;
	gl->addWidget(runningItem = new QLabel(""), l, 1); ++l;
	gl->addWidget(addressItem = new QLabel(""), l, 1); ++l;
	gl->addWidget(firmwareItem = new QLabel("n/a"), l, 1); ++l;
	gl->addWidget(indexItem = new QComboBox(), l, 1); ++l;
	gl->addWidget(selectAcc = new QCheckBox("Raw Accelerometer"), l, 1); ++l;
	gl->addWidget(selectMag = new QCheckBox("Raw Magnetometer"), l, 1); ++l;
	gl->addWidget(selectGyro = new QCheckBox("Angular velocity"), l, 1); ++l;
	gl->addWidget(selectQuaternion = new QCheckBox("Quaternion"), l, 1); ++l;
	gl->addWidget(selectEuler = new QCheckBox("Euler angle"), l, 1); ++l;
	gl->addWidget(selectLinAcc = new QCheckBox("Lin. acceleration"), l, 1); ++l;	
	gl->addWidget(samplingRateCombo = new QComboBox(), l, 1); ++l;	
	gl->addWidget(gyrRangeCombo = new QComboBox(), l, 1); ++l;
	gl->addWidget(accRangeCombo = new QComboBox(), l, 1); ++l;
	gl->addWidget(magRangeCombo = new QComboBox(), l, 1); ++l;
	gl->addWidget(parameterSetCombo = new QComboBox(), l, 1); ++l;
	gl->addWidget(filterModeCombo = new QComboBox(), l, 1); ++l;
	gl->addWidget(thresholdEnableCombo = new QComboBox(), l, 1); ++l;
	gl->addWidget(gyrAutocalibrationCombo = new QComboBox(), l, 1); ++l;
	
	connect(selectAcc, SIGNAL(stateChanged(int)), this, SLOT(updateSelectAcc(int)));
	connect(selectMag, SIGNAL(stateChanged(int)), this, SLOT(updateSelectMag(int)));
	connect(selectGyro, SIGNAL(stateChanged(int)), this, SLOT(updateSelectGyro(int)));
	connect(selectQuaternion, SIGNAL(stateChanged(int)), this, SLOT(updateSelectQuaternion(int)));
	connect(selectEuler, SIGNAL(stateChanged(int)), this, SLOT(updateSelectEuler(int)));
	connect(selectLinAcc, SIGNAL(stateChanged(int)), this, SLOT(updateSelectLinAcc(int)));
	
	for (int i=1; i < 33; i++) indexItem->addItem(QString("%1").arg(i));
	connect(indexItem, SIGNAL(currentIndexChanged(int)), this, SLOT(updateOpenMATIndex(int)));

	samplingRateCombo->addItem(QString("5 Hz"));
	samplingRateCombo->addItem(QString("10 Hz"));
	samplingRateCombo->addItem(QString("50 Hz"));
	samplingRateCombo->addItem(QString("100 Hz"));
	samplingRateCombo->addItem(QString("200 Hz"));	
	if (deviceType == DEVICE_LPMS_U || deviceType == DEVICE_LPMS_C) {
		samplingRateCombo->addItem(QString("500 Hz"));	
	}
	connect(samplingRateCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(updatesamplingRate(int)));
	
	parameterSetCombo->addItem(QString("Weak"));
	parameterSetCombo->addItem(QString("Medium"));
	parameterSetCombo->addItem(QString("Strong"));
	parameterSetCombo->addItem(QString("Dynamic"));
	connect(parameterSetCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(updateFilterPreset(int)));

	filterModeCombo->addItem(QString("Gyr only"));
	filterModeCombo->addItem(QString("Gyr + Acc"));
	filterModeCombo->addItem(QString("Gyr + Acc + Mag"));
	// filterModeCombo->addItem(QString("G+A+M switching"));
	connect(filterModeCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(updateFilterMode(int)));	
	
	/* magThreshEnableCombo->addItem(QString("Disable"));
	magThreshEnableCombo->addItem(QString("5 uT"));
	magThreshEnableCombo->addItem(QString("10 uT"));
	magThreshEnableCombo->addItem(QString("20 uT")); */
	
	thresholdEnableCombo->addItem(QString("Disable"));	
	thresholdEnableCombo->addItem(QString("Enable"));	
	connect(thresholdEnableCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(updateGyrThresholdEnable(int)));	
	
	gyrAutocalibrationCombo->addItem(QString("Disable"));	
	gyrAutocalibrationCombo->addItem(QString("Enable"));	
	connect(gyrAutocalibrationCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(updateGyrAutocalibration(int)));

	accRangeCombo->addItem(QString("2G"));
	accRangeCombo->addItem(QString("4G"));
	accRangeCombo->addItem(QString("8G"));
	accRangeCombo->addItem(QString("16G"));
	connect(accRangeCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(updateAccRange(int)));		
	
	magRangeCombo->addItem(QString("130 uT"));
	magRangeCombo->addItem(QString("190 uT"));
	magRangeCombo->addItem(QString("250 uT"));
	magRangeCombo->addItem(QString("400 uT"));
	magRangeCombo->addItem(QString("560 uT"));
	magRangeCombo->addItem(QString("810 uT"));
	connect(magRangeCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(updateMagRange(int)));		

	gyrRangeCombo->addItem(QString("250 dps"));
	gyrRangeCombo->addItem(QString("500 dps"));
	gyrRangeCombo->addItem(QString("2000 dps"));
	connect(gyrRangeCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(updateGyrRange(int)));
	
	if (deviceType == DEVICE_LPMS_U || deviceType == DEVICE_LPMS_C) {
		gl->addWidget(new QLabel("CAN protocol"), l, 0);
		gl->addWidget(canProtocolCombo = new QComboBox(), l, 1); ++l;
		
		canProtocolCombo->addItem(QString("LPBus"));
		canProtocolCombo->addItem(QString("CANOpen"));
		canProtocolCombo->addItem(QString("Custom 1"));
		canProtocolCombo->addItem(QString("Custom 2"));
		
		connect(canProtocolCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(updateCanProtocol(int)));		
		
		gl->addWidget(new QLabel("CAN baudrate"), l, 0);
		gl->addWidget(canBaudrateCombo = new QComboBox(), l, 1); ++l;	
	
		canBaudrateCombo->addItem(QString("1 MBit/s"));
		canBaudrateCombo->addItem(QString("500 KBit/s"));
		canBaudrateCombo->addItem(QString("250 KBit/s"));
		canBaudrateCombo->addItem(QString("125 KBit/s"));
		
		connect(canBaudrateCombo, SIGNAL(currentIndexChanged(int)), this, SLOT(updateCanBaudrate(int)));		
	}
	
	setLayout(gl);
	
	updateData();
	
	tree->setItemWidget(subTreeItem, 0, this);	
}

void SensorGuiContainer::updateData(void) {	
	int i;
	string s;
	
	indexItem->blockSignals(true);	
	parameterSetCombo->blockSignals(true);	
	thresholdEnableCombo->blockSignals(true);	
	accRangeCombo->blockSignals(true);	
	magRangeCombo->blockSignals(true);	
	gyrRangeCombo->blockSignals(true);	
	filterModeCombo->blockSignals(true);	
	selectQuaternion->blockSignals(true);
	selectEuler->blockSignals(true);
	selectLinAcc->blockSignals(true);
	selectGyro->blockSignals(true);
	selectAcc->blockSignals(true);
	selectMag->blockSignals(true);


	if (deviceType == DEVICE_LPMS_U || deviceType == DEVICE_LPMS_C) {
		canProtocolCombo->blockSignals(false);	
		canBaudrateCombo->blockSignals(false);	
	}

	sensor->getConfigurationPrm(PRM_DEVICE_TYPE, &i);
	switch (i) {
	case DEVICE_LPMS_B:
		treeItem->setText(0, QString("LPMS-B"));
	break;

	case DEVICE_LPMS_U:
		treeItem->setText(0, QString("LPMS-U"));
	break;	

	case DEVICE_LPMS_C:
		treeItem->setText(0, QString("LPMS-C"));
	break;		
	}	
	
	sensor->getConfigurationPrm(PRM_DEVICE_ID, &s);	
	addressItem->setText(QString(s.c_str()));	

	sensor->getConfigurationPrm(PRM_OPENMAT_ID, &i);		
	indexItem->setCurrentIndex(i-1);	

	sensor->getConfigurationPrm(PRM_FIRMWARE_VERSION, &s);		
	firmwareItem->setText(QString(s.c_str()));
	
	sensor->getConfigurationPrm(PRM_PARAMETER_SET, &i);	
	switch (i) {
	case SELECT_IMU_SLOW:
		parameterSetCombo->setCurrentIndex(0);	
	break;

	case SELECT_IMU_MEDIUM:
		parameterSetCombo->setCurrentIndex(1);		
	break;
		
	case SELECT_IMU_FAST:
		parameterSetCombo->setCurrentIndex(2);		
	break;		
	
	case SELECT_IMU_DYNAMIC:
		parameterSetCombo->setCurrentIndex(3);		
	break;
	}
	
	sensor->getConfigurationPrm(PRM_GYR_THRESHOLD_ENABLE, &i);		
	switch (i) {
	case SELECT_IMU_GYR_THRESH_ENABLE:
		thresholdEnableCombo->setCurrentIndex(1);	
	break;

	case SELECT_IMU_GYR_THRESH_DISABLE:
		thresholdEnableCombo->setCurrentIndex(0);		
	break;
	}	

	sensor->getConfigurationPrm(PRM_GYR_AUTOCALIBRATION, &i);		
	switch (i) {
	case SELECT_GYR_AUTOCALIBRATION_ENABLE:
		gyrAutocalibrationCombo->setCurrentIndex(1);	
	break;

	case SELECT_GYR_AUTOCALIBRATION_DISABLE:
		gyrAutocalibrationCombo->setCurrentIndex(0);		
	break;
	}
	
	sensor->getConfigurationPrm(PRM_FILTER_MODE, &i);		
	switch (i) {
	case SELECT_FM_GYRO_ONLY:
		filterModeCombo->setCurrentIndex(0);
	break;

	case SELECT_FM_GYRO_ACC:
		filterModeCombo->setCurrentIndex(1);	
	break;
		
	case SELECT_FM_GYRO_ACC_MAG_NS:
		filterModeCombo->setCurrentIndex(2);	
	break;

	case SELECT_FM_GYRO_ACC_MAG_SW:
		filterModeCombo->setCurrentIndex(3);	
	break;		
	}	
	
	switch (sensor->getSensorStatus()) {
	case 0:
		runningItem->setText("<font color='black'>stopped</font>");
	break;

	case 1:
		runningItem->setText("<font color='green'>started</font>");
	break;

	case 2:
		runningItem->setText("<font color='blue'>calibrating..</font>");
	break;	

	case 3:
		runningItem->setText("<font color='blue'>error..</font>");
	break;	

	case 4:
		runningItem->setText("<font color='blue'>uploading..</font>");
	break;	
	}
		
	switch (sensor->getConnectionStatus()) {
	case 0:
	break;
		
	case 1:
		statusItem->setText("<font color='green'>OK</font>");
	break;
		
	case 2:
		statusItem->setText("<font color='blue'>in progress</font>");
	break;	
		
	case 3:
		statusItem->setText("<font color='red'>failed</font>");
	break;		

	case 4:
		statusItem->setText("<font color='red'>interrupted</font>");
	break;
	}	
	
	sensor->getConfigurationPrm(PRM_GYR_RANGE, &i);	
	switch (i) {
	case SELECT_GYR_RANGE_250DPS:		
		gyrRangeCombo->setCurrentIndex(0);
	break;
		
	case SELECT_GYR_RANGE_500DPS:
		gyrRangeCombo->setCurrentIndex(1);
	break;
		
	case SELECT_GYR_RANGE_2000DPS:
		gyrRangeCombo->setCurrentIndex(2);
	break;
	}
	
	sensor->getConfigurationPrm(PRM_ACC_RANGE, &i);		
	switch (i) {
	case SELECT_ACC_RANGE_2G:		
		accRangeCombo->setCurrentIndex(0);
	break;
		
	case SELECT_ACC_RANGE_4G:
		accRangeCombo->setCurrentIndex(1);
	break;
		
	case SELECT_ACC_RANGE_8G:
		accRangeCombo->setCurrentIndex(2);
	break;
		
	case SELECT_ACC_RANGE_16G:
		accRangeCombo->setCurrentIndex(3);
	break;			
	}	
	
	sensor->getConfigurationPrm(PRM_MAG_RANGE, &i);		
	switch (i) {
	case SELECT_MAG_RANGE_130UT:		
		magRangeCombo->setCurrentIndex(0);
	break;
		
	case SELECT_MAG_RANGE_190UT:
		magRangeCombo->setCurrentIndex(1);
	break;
		
	case SELECT_MAG_RANGE_250UT:
		magRangeCombo->setCurrentIndex(2);
	break;
		
	case SELECT_MAG_RANGE_400UT:
		magRangeCombo->setCurrentIndex(3);
	break;	

	case SELECT_MAG_RANGE_560UT:
		magRangeCombo->setCurrentIndex(4);
	break;
		
	case SELECT_MAG_RANGE_810UT:
		magRangeCombo->setCurrentIndex(5);
	break;		
	}
	
	if (deviceType == DEVICE_LPMS_U || deviceType == DEVICE_LPMS_C) {
		sensor->getConfigurationPrm(PRM_CAN_STREAM_FORMAT, &i);		
		switch (i) {
		case SELECT_STREAM_CAN_LPBUS:		
			canProtocolCombo->setCurrentIndex(0);
		break;
	
		case SELECT_STREAM_CAN_OPEN:
			canProtocolCombo->setCurrentIndex(1);
		break;	
	
		case SELECT_STREAM_CAN_CUSTOM1:
			canProtocolCombo->setCurrentIndex(2);
		break;

		case SELECT_STREAM_CAN_CUSTOM2:
			canProtocolCombo->setCurrentIndex(3);
		break;
		}	

		sensor->getConfigurationPrm(PRM_CAN_BAUDRATE, &i);		
		switch (i) {
		case SELECT_CAN_BAUDRATE_1000KBPS:		
			canBaudrateCombo->setCurrentIndex(0);
		break;
		
		case SELECT_CAN_BAUDRATE_500KBPS:
			canBaudrateCombo->setCurrentIndex(1);
		break;
		
		case SELECT_CAN_BAUDRATE_250KBPS:
			canBaudrateCombo->setCurrentIndex(2);
		break;
		
		case SELECT_CAN_BAUDRATE_125KBPS:
			canBaudrateCombo->setCurrentIndex(3);
		break;
		}	
	}

	sensor->getConfigurationPrm(PRM_SAMPLING_RATE, &i);		
	switch (i) {
	case SELECT_STREAM_FREQ_5HZ:		
		samplingRateCombo->setCurrentIndex(0);
	break;		
	
	case SELECT_STREAM_FREQ_10HZ:		
		samplingRateCombo->setCurrentIndex(1);
	break;	
	
	case SELECT_STREAM_FREQ_50HZ:		
		samplingRateCombo->setCurrentIndex(2);
	break;
		
	case SELECT_STREAM_FREQ_100HZ:
		samplingRateCombo->setCurrentIndex(3);
	break;
		
	case SELECT_STREAM_FREQ_200HZ:
		samplingRateCombo->setCurrentIndex(4);
	break;
	
	case SELECT_STREAM_FREQ_500HZ:
		samplingRateCombo->setCurrentIndex(5);
	break;	
	}	
		
	sensor->getConfigurationPrm(PRM_SELECT_DATA, &i);
	if ((i & SELECT_LPMS_ACC_OUTPUT_ENABLED) != 0) {
		selectAcc->setCheckState(Qt::Checked);
	} else {
		selectAcc->setCheckState(Qt::Unchecked);
	}
	
	if ((i & SELECT_LPMS_MAG_OUTPUT_ENABLED) != 0) {
		selectMag->setCheckState(Qt::Checked);
	} else {
		selectMag->setCheckState(Qt::Unchecked);
	}
	
	if ((i & SELECT_LPMS_GYRO_OUTPUT_ENABLED) != 0) {
		selectGyro->setCheckState(Qt::Checked);
	} else {
		selectGyro->setCheckState(Qt::Unchecked);
	}

	if ((i & SELECT_LPMS_QUAT_OUTPUT_ENABLED) != 0) {
		selectQuaternion->setCheckState(Qt::Checked);
	} else {
		selectQuaternion->setCheckState(Qt::Unchecked);
	}
	
	if ((i & SELECT_LPMS_EULER_OUTPUT_ENABLED) != 0) {
		selectEuler->setCheckState(Qt::Checked);
	} else {
		selectEuler->setCheckState(Qt::Unchecked);
	}	
		
	if ((i & SELECT_LPMS_LINACC_OUTPUT_ENABLED) != 0) {
		selectLinAcc->setCheckState(Qt::Checked);
	} else {
		selectLinAcc->setCheckState(Qt::Unchecked);
	}

	selectAcc->blockSignals(false);
	selectMag->blockSignals(false);	
	selectQuaternion->blockSignals(false);
	selectEuler->blockSignals(false);
	selectLinAcc->blockSignals(false);
	selectGyro->blockSignals(false);
	indexItem->blockSignals(false);	
	parameterSetCombo->blockSignals(false);	
	thresholdEnableCombo->blockSignals(false);	
	accRangeCombo->blockSignals(false);	
	magRangeCombo->blockSignals(false);	
	gyrRangeCombo->blockSignals(false);
	filterModeCombo->blockSignals(false);

	if (deviceType == DEVICE_LPMS_U || deviceType == DEVICE_LPMS_C) {
		canProtocolCombo->blockSignals(false);	
		canBaudrateCombo->blockSignals(false);	
	}
}

void SensorGuiContainer::updateOpenMATIndex(int i)
{
	sensor->setConfigurationPrm(PRM_OPENMAT_ID, indexItem->currentIndex()+1);
}

void SensorGuiContainer::updateGyrThresholdEnable(int i)
{
	switch (thresholdEnableCombo->currentIndex()) {
	case 1:
		sensor->setConfigurationPrm(PRM_GYR_THRESHOLD_ENABLE, SELECT_IMU_GYR_THRESH_ENABLE);
		break;
		
	case 0:
		sensor->setConfigurationPrm(PRM_GYR_THRESHOLD_ENABLE, SELECT_IMU_GYR_THRESH_DISABLE);
		break;
	}
}

void SensorGuiContainer::updateGyrAutocalibration(int i)
{
	switch (gyrAutocalibrationCombo->currentIndex()) {
	case 1:
		sensor->setConfigurationPrm(PRM_GYR_AUTOCALIBRATION, SELECT_GYR_AUTOCALIBRATION_ENABLE);
	break;
		
	case 0:
		sensor->setConfigurationPrm(PRM_GYR_AUTOCALIBRATION, SELECT_GYR_AUTOCALIBRATION_DISABLE);
	break;
	}
}

void SensorGuiContainer::updateFilterPreset(int i)
{
	switch (parameterSetCombo->currentIndex()) {
	case 0:
		sensor->setConfigurationPrm(PRM_PARAMETER_SET, SELECT_IMU_SLOW);
	break;
		
	case 1:
		sensor->setConfigurationPrm(PRM_PARAMETER_SET, SELECT_IMU_MEDIUM);
	break;
		
	case 2:
		sensor->setConfigurationPrm(PRM_PARAMETER_SET, SELECT_IMU_FAST);
	break;	
	
	case 3:
		sensor->setConfigurationPrm(PRM_PARAMETER_SET, SELECT_IMU_DYNAMIC);
	break;	
	}
}

void SensorGuiContainer::updateFilterMode(int i)
{
	switch (filterModeCombo->currentIndex()) {
	case 0:
		sensor->setConfigurationPrm(PRM_FILTER_MODE, SELECT_FM_GYRO_ONLY);
	break;
		
	case 1:
		sensor->setConfigurationPrm(PRM_FILTER_MODE, SELECT_FM_GYRO_ACC);
	break;
		
	case 2:
		sensor->setConfigurationPrm(PRM_FILTER_MODE, SELECT_FM_GYRO_ACC_MAG_NS);
	break;

	case 3:
		sensor->setConfigurationPrm(PRM_FILTER_MODE, SELECT_FM_GYRO_ACC_MAG_SW);
	break;
	}
}

void SensorGuiContainer::updateGyrRange(int i)
{	
	switch (gyrRangeCombo->currentIndex()) {
	case 0:
		sensor->setConfigurationPrm(PRM_GYR_RANGE, SELECT_GYR_RANGE_250DPS);
	break;
		
	case 1:
		sensor->setConfigurationPrm(PRM_GYR_RANGE, SELECT_GYR_RANGE_500DPS);
	break;
		
	case 2:
		sensor->setConfigurationPrm(PRM_GYR_RANGE, SELECT_GYR_RANGE_2000DPS);
	break;
	}
}	

void SensorGuiContainer::updateAccRange(int i)
{	
	switch (accRangeCombo->currentIndex()) {
	case 0:
		sensor->setConfigurationPrm(PRM_ACC_RANGE, SELECT_ACC_RANGE_2G);
	break;
		
	case 1:
		sensor->setConfigurationPrm(PRM_ACC_RANGE, SELECT_ACC_RANGE_4G);
	break;
		
	case 2:
		sensor->setConfigurationPrm(PRM_ACC_RANGE, SELECT_ACC_RANGE_8G);
	break;

	case 3:
		sensor->setConfigurationPrm(PRM_ACC_RANGE, SELECT_ACC_RANGE_16G);
	break;
	}	
}

void SensorGuiContainer::updateMagRange(int i)
{	
	switch (magRangeCombo->currentIndex()) {
	case 0:
		sensor->setConfigurationPrm(PRM_MAG_RANGE, SELECT_MAG_RANGE_130UT);
	break;
		
	case 1:
		sensor->setConfigurationPrm(PRM_MAG_RANGE, SELECT_MAG_RANGE_190UT);
	break;
		
	case 2:
		sensor->setConfigurationPrm(PRM_MAG_RANGE, SELECT_MAG_RANGE_250UT);
	break;

	case 3:
		sensor->setConfigurationPrm(PRM_MAG_RANGE, SELECT_MAG_RANGE_400UT);
	break;
		
	case 4:
		sensor->setConfigurationPrm(PRM_MAG_RANGE, SELECT_MAG_RANGE_560UT);
	break;

	case 5:
		sensor->setConfigurationPrm(PRM_MAG_RANGE, SELECT_MAG_RANGE_810UT);
	break;		
	}
}

void SensorGuiContainer::updateCanProtocol(int i)
{	
	switch (canProtocolCombo->currentIndex()) {
	case 0:
		sensor->setConfigurationPrm(PRM_CAN_STREAM_FORMAT, SELECT_STREAM_CAN_LPBUS);
	break;
		
	case 1:
		sensor->setConfigurationPrm(PRM_CAN_STREAM_FORMAT, SELECT_STREAM_CAN_OPEN);
	break;
	
	case 2:
		sensor->setConfigurationPrm(PRM_CAN_STREAM_FORMAT, SELECT_STREAM_CAN_CUSTOM1);
	break;
	
	case 3:
		sensor->setConfigurationPrm(PRM_CAN_STREAM_FORMAT, SELECT_STREAM_CAN_CUSTOM2);
	break;
	}
}

void SensorGuiContainer::updateCanBaudrate(int i)
{	
	switch (canBaudrateCombo->currentIndex()) {
	case 0:
		sensor->setConfigurationPrm(PRM_CAN_BAUDRATE, SELECT_CAN_BAUDRATE_1000KBPS);
	break;

	case 1:
		sensor->setConfigurationPrm(PRM_CAN_BAUDRATE, SELECT_CAN_BAUDRATE_500KBPS);
	break;	
	
	case 2:
		sensor->setConfigurationPrm(PRM_CAN_BAUDRATE, SELECT_CAN_BAUDRATE_250KBPS);
	break;
		
	case 3:
		sensor->setConfigurationPrm(PRM_CAN_BAUDRATE, SELECT_CAN_BAUDRATE_125KBPS);
	break;
	}
}

void SensorGuiContainer::updatesamplingRate(int i)
{	
	switch (samplingRateCombo->currentIndex()) {
	case 0:
		sensor->setConfigurationPrm(PRM_SAMPLING_RATE, SELECT_STREAM_FREQ_5HZ);
	break;
	
	case 1:
		sensor->setConfigurationPrm(PRM_SAMPLING_RATE, SELECT_STREAM_FREQ_10HZ);
	break;

	case 2:
		sensor->setConfigurationPrm(PRM_SAMPLING_RATE, SELECT_STREAM_FREQ_50HZ);
	break;
		
	case 3:
		sensor->setConfigurationPrm(PRM_SAMPLING_RATE, SELECT_STREAM_FREQ_100HZ);
	break;
		
	case 4:
		sensor->setConfigurationPrm(PRM_SAMPLING_RATE, SELECT_STREAM_FREQ_200HZ);
	break;
	
	case 5:
		sensor->setConfigurationPrm(PRM_SAMPLING_RATE, SELECT_STREAM_FREQ_500HZ);
	break;
	}
}

SensorGuiContainer::~SensorGuiContainer(void)
{
}

LpmsSensorI* SensorGuiContainer::getSensor(void)
{
	return sensor;
}

void SensorGuiContainer::updateSelectGyro(int i)
{
	int p = 0;

	sensor->getConfigurationPrm(PRM_SELECT_DATA, &p);
	
	if (i == Qt::Checked) {
		p |= SELECT_LPMS_GYRO_OUTPUT_ENABLED;
	} else {
		p &= ~SELECT_LPMS_GYRO_OUTPUT_ENABLED;	
	}
	
	sensor->setConfigurationPrm(PRM_SELECT_DATA, p);
}

void SensorGuiContainer::updateSelectQuaternion(int i)
{
	int p = 0;

	sensor->getConfigurationPrm(PRM_SELECT_DATA, &p);
	
	if (i == Qt::Checked) {
		p |= SELECT_LPMS_QUAT_OUTPUT_ENABLED;
	} else {
		p &= ~SELECT_LPMS_QUAT_OUTPUT_ENABLED;	
	}
	
	sensor->setConfigurationPrm(PRM_SELECT_DATA, p);
}

void SensorGuiContainer::updateSelectEuler(int i)
{
	int p = 0;

	sensor->getConfigurationPrm(PRM_SELECT_DATA, &p);	

	if (i == Qt::Checked) {
		p |= SELECT_LPMS_EULER_OUTPUT_ENABLED;
	} else {
		p &= ~SELECT_LPMS_EULER_OUTPUT_ENABLED;	
	}

	sensor->setConfigurationPrm(PRM_SELECT_DATA, p);	
}

void SensorGuiContainer::updateSelectLinAcc(int i)
{
	int p = 0;
	
	sensor->getConfigurationPrm(PRM_SELECT_DATA, &p);	
	
	if (i == Qt::Checked) {
		p |= SELECT_LPMS_LINACC_OUTPUT_ENABLED;
	} else {
		p &= ~SELECT_LPMS_LINACC_OUTPUT_ENABLED;	
	}
	
	sensor->setConfigurationPrm(PRM_SELECT_DATA, p);
}

void SensorGuiContainer::updateSelectAcc(int i)
{
	int p = 0;
	
	sensor->getConfigurationPrm(PRM_SELECT_DATA, &p);	
	
	if (i == Qt::Checked) {
		p |= SELECT_LPMS_ACC_OUTPUT_ENABLED;
	} else {
		p &= ~SELECT_LPMS_ACC_OUTPUT_ENABLED;	
	}
	
	sensor->setConfigurationPrm(PRM_SELECT_DATA, p);
}

void SensorGuiContainer::updateSelectMag(int i)
{
	int p = 0;
	
	sensor->getConfigurationPrm(PRM_SELECT_DATA, &p);	
	
	if (i == Qt::Checked) {
		p |= SELECT_LPMS_MAG_OUTPUT_ENABLED;
	} else {
		p &= ~SELECT_LPMS_MAG_OUTPUT_ENABLED;	
	}
	
	sensor->setConfigurationPrm(PRM_SELECT_DATA, p);
}