#ifndef SENSOR_GUI_CONTAINER
#define SENSOR_GUI_CONTAINER 

#include <QTreeWidgetItem>
#include <QComboBox>
#include <QLabel>
#include <QGridLayout>
#include <QTreeWidgetItem>
#include <QTreeWidget>
#include <QCheckbox.h>

#include "LpmsSensorI.h"
#include "LpmsDefinitions.h"

#include <list>
#include <string>
#include <iostream>
using namespace std;

/*!
	\brief	Container for GUI elements related to one sensor unit.
	
	Its contents are used to build the sensor list tree in the control window.
*/
class SensorGuiContainer : QWidget 
{
	Q_OBJECT

public:
	QTreeWidgetItem* treeItem;
	QLabel* addressItem;
	QLabel* deviceTypeItem;		
	QLabel* parameterSetItem;
	QLabel* nameItem;
	QLabel* statusItem;
	QComboBox* indexItem;
	QLabel* runningItem;
	QLabel* firmwareItem;
	QLabel* thresholdEnableItem;
	QLabel* filterModeItem;
	QComboBox* filterModeCombo;
	QComboBox* parameterSetCombo;
	QComboBox* thresholdEnableCombo;
	QComboBox* gyrRangeCombo;
	QComboBox* accRangeCombo;
	QComboBox* magRangeCombo;
	QComboBox* gyrAutocalibrationCombo;
	QComboBox* canProtocolCombo;
	QComboBox* canBaudrateCombo;
	QComboBox* samplingRateCombo;
	QComboBox* magThreshEnableCombo;
	QCheckBox* selectQuaternion;
	QCheckBox* selectEuler;
	QCheckBox* selectLinAcc;
	QCheckBox* selectGyro;
	QCheckBox* selectAcc;
	QCheckBox* selectMag;
	int openMatId;
	LpmsSensorI* sensor;	
	
	SensorGuiContainer(LpmsSensorI* sensor, QTreeWidget* tree);
	~SensorGuiContainer(void);
	LpmsSensorI* getSensor(void);
	int deviceType;
	
public slots:
	void updateData(void);
	void updateGyrThresholdEnable(int i);
	void updateFilterPreset(int i);
	void updateFilterMode(int i);
	void updateGyrRange(int i);
	void updateAccRange(int i);
	void updateMagRange(int i);
	void updateOpenMATIndex(int i);
	void updateGyrAutocalibration(int i);
	void updateCanBaudrate(int i);	
	void updateCanProtocol(int i);
	void updatesamplingRate(int i);
	void updateSelectGyro(int i);	
	void updateSelectQuaternion(int i);
	void updateSelectEuler(int i);
	void updateSelectLinAcc(int i);
	void updateSelectAcc(int i);
	void updateSelectMag(int i);
};

#endif