/****************************************************************************
**
** Copyright (C) 2011 LP-Research
** All rights reserved.
** Contact: LP-Research (klaus@lp-research.com)
**
** This file is part of the Open Motion Analysis Toolkit (OpenMAT).
**
** OpenMAT is free software: you can redistribute it and/or modify it under 
** the terms of the GNU General Public License as published by the Free 
** Software Foundation, either version 3 of the License, or (at your option) 
** any later version.
** 
** OpenMAT is distributed in the hope that it will be useful, but WITHOUT 
** ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or 
** FITNESS FOR A PARTICULAR PURPOSE. See the GNU \ General Public License 
** for more details.
** 
** You should have received a copy of the GNU General Public License along 
** with the OpenMAT library. If not, see <http://www.gnu.org/licenses/>.
**
****************************************************************************/

#ifndef THREE_D_WINDOW
#define THREE_D_WINDOW

#include <QtGui>
#include <QGLWidget>

#include "ImuData.h"
#include "ObjFileParser.h"
#include "LpmsDefinitions.h"

#include <string>
#include <iostream>
using namespace std;

#include <Eigen/Dense>	

class QGLShaderProgram;

/*!
	\brief	Shows the 3d-cube visualization of the sampled IMU data.
*/
class ThreeDWindow : public QGLWidget
{
    Q_OBJECT

public:	
	int activeOpenMatId;
	QPoint lastPos;
	int xRot;
	int yRot;
	int zRot;
	int xSRot;
	int ySRot;
	int zSRot;
	ObjFileParser caseObj;
	float lpmsCaseScale;
	Eigen::Matrix3f rM;
	ImuData imuData;
	Eigen::Vector3f fieldMap[ABSMAXPITCH][ABSMAXROLL][ABSMAXYAW];
	Eigen::Vector3f hardIronOffset;
	Eigen::Matrix3f softIronMatrix;

    ThreeDWindow(QWidget *parent = 0, QGLWidget *shareWidget = 0);
    ~ThreeDWindow();
    QSize minimumSizeHint() const;
    QSize sizeHint() const;
	void drawAxes(void);
	void setActiveLpms(int openMatId);
	void drawPointCube(void);
    void initializeGL();
    void resizeGL(int width, int height);
	void drawTri(Eigen::Vector3f p0, 
		Eigen::Vector3f p1, 
		Eigen::Vector3f p2,
		bool fill);
	void drawQuad(Eigen::Vector3f p0, 
		Eigen::Vector3f p1, 
		Eigen::Vector3f p2, 
		Eigen::Vector3f p3, 
		bool fill);
	void drawFloor(void);
	void drawLpmsCase(void);
	void drawBackground(void);	
	void drawCylinder(float l, float r, Eigen::Matrix3f m);
	void drawCone(float l, float r, Eigen::Matrix4f T4, std::string tipText);
	void zeroImuData(ImuData* id);
	void drawGridCube(void);
	void drawSphere(Eigen::Vector3f p, float r);
	void drawFieldMap(void);

signals:
    void clicked();

public slots:	
	void updateQuaternion(ImuData imuData);
	void updateFieldMap(float magField[ABSMAXPITCH][ABSMAXROLL][ABSMAXYAW][3],
		float hardIronOffset[3], float softIronMatrix[3][3]);
	void updateWindow(void);
	void rotateBy(int xAngle, int yAngle, int zAngle);
	void rotateSceneBy(int xAngle, int yAngle, int zAngle);
	void mousePressEvent(QMouseEvent *event);
	void mouseMoveEvent(QMouseEvent *event);	
    void paintGL();	
};

#endif
